set

Function set 

pub fn set(
    conditions: &mut Vec<Condition>,
    desired: DesiredCondition,
) -> ConditionChange
Sets the condition of desired.type_ to desired, adding it if it is not already present, and returns what changed.

Its lastTransitionTime is set to the current time if the condition is added or its status changes, and otherwise kept.