pub fn find_observed<'a>(
conditions: &'a [Condition],
type_: &str,
generation: Option<i64>,
) -> Option<&'a Condition>Expand description
Returns the condition of the given type, if present and determined from
the given generation of its resource (which should be that resource’s
current metadata.generation).
Use this rather than find when reading the conditions of a resource
that another controller maintains: until that controller has reconciled
the latest change to the resource’s spec, its conditions describe the
previous spec.