pub enum Error<E: Error + 'static> {
ControllerError(E),
FinalizerError(Error<E>),
}Expand description
An error from a reconciliation pass, as passed to
Context::error_action and Context::failure_event.
Variants
ControllerError(E)
Context::apply returned an error, and the context has no
finalizer.
FinalizerError(Error<E>)
The context has a finalizer, and either Context::apply or
Context::cleanup returned an error (wrapped as
ApplyFailed or
CleanupFailed), or
adding or removing the finalizer failed.
Implementations
Sourceimpl<E: Error + 'static> Error<E>
impl<E: Error + 'static> Error<E>
Sourcepub fn display_chain(&self) -> String
pub fn display_chain(&self) -> String
Renders this error followed by each of its causes, separated by
": ", as used in the note of the default
failure event.